<?php
// This file is part of the Laerdal EcoHub LTI Source sub-plugin.
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

defined('MOODLE_INTERNAL') || die();

require_once('storage.php');
require_once('mapping.php');

/**
 * Data loader class for configuration settings cache.
 *
 * This is essentially a cache wrapper for configuration settings for easy access to the settings.
 * This is an extension class of cache_application.
 *
 * @copyright 2018 Laerdal Medical A/S
 * @author Martin Rieva
 * @package ecohub
 */
class ecohub_settings_cache_data_loader extends cache_application {
    protected $filestorage = null;
    protected $lastfilestamp = null;
    protected $dbstorage = null;

    /**
     * Overrides the cache construct method.
     *
     * @param cache_definition $definition
     * @param cache_store $store
     * @param cache_loader|cache_data_source $loader
     * @throws file_exception
     */
    public function __construct(cache_definition $definition, cache_store $store, $loader = null) {
        $this->filestorage = new ltisource_ecohub_file_storage(
            new ltisource_ecohub_config_parser(),
            cache::make('ltisource_ecohub', 'configfile'));
        $this->dbstorage = new ltisource_ecohub_db_storage();

        parent::__construct($definition, $store, $loader);
    }

    /**
     * Retrieves the value for the given key from the cache.
     *
     * @param string|int $key The key for the data being requested.
     * @param int $strictness One of IGNORE_MISSING | MUST_EXIST
     * @return mixed|false The data from the cache or false if the key did not exist within the cache.
     */
    public function get($key, $strictness = IGNORE_MISSING) {
        $value = false;
        if ($this->lastfilestamp === null) {
            // First time call after init and config file might also be changed.
            $dbdataset = $this->dbstorage->get();
            if (empty($dbdataset) ||
                    !array_key_exists('_modified', $dbdataset) ||
                    $dbdataset['_modified'] < $this->filestorage->last_modified()) {
                $this->copy_file_config_to_db($dbdataset !== false, false);
            } else {
                parent::set_many($dbdataset);
                if (array_key_exists('_modified', $dbdataset)) {
                    $this->lastfilestamp = $dbdataset['_modified'];
                }
            }
            cache_helper::purge_by_event('ecohubconfigchanged');
            $value = parent::get($key);
        } else if ($this->lastfilestamp < $this->filestorage->last_modified()) {
            // Config file has changed so we need to reload.
            $this->copy_file_config_to_db(true, true);
        }
        // Return value if cached.
        if ($value === false) {
            $value = parent::get($key, $strictness);
        }
        // Otherwise load from db and cache it.
        if ($value === false) {
            $value = $this->dbstorage->get($key);
            if ($value !== false) {
                parent::set($key, $value);
            }
        }
        return $value;
    }

    /**
     * Sends a key => value pair to the cache.
     *
     * @param string|int $key The key for the data being requested.
     * @param mixed $data The data to set against the key.
     * @return bool True on success, false otherwise.
     */
    public function set($key, $data) {
        $ecohubmap = ltisource_ecohub_mapping::get_instance();
        if (is_array($data)) {
            foreach ($data as $param => $mapentry) {
                if ($ecohubmap->expand_map_entry($mapentry) === false) {
                    return false;
                }
            }
        } else if ($ecohubmap->expand_map_entry($data) === false) {
            return false;
        }
        if ($this->dbstorage->set($key, $data) === false) {
            return false;
        }
        if (is_array($data)) {
            $rootdata = parent::get($key);
            foreach ($data as $param => $mapentry) {
                $rootdata[$param] = $mapentry;
            }
            parent::set($key, $rootdata);
        } else {
            parent::set($key, $data);
        }
        return true;
    }

    /**
     * Purges the cache store, and loader if there is one.
     *
     * @return bool True on success, false otherwise
     */
    public function purge() {
        if (parent::purge()) {
            $this->lastfilestamp = null;
        }
        return true;
    }

    protected function copy_file_config_to_db($purgedb = false, $purgecache = false) {
        $filedataset = $this->filestorage->get();
        if ($filedataset !== false) {
            if ($purgedb) {
                $this->dbstorage->purge();
            }
            $this->dbstorage->set($filedataset);
            $this->lastfilestamp = $this->filestorage->last_modified();
            $this->dbstorage->set('_modified', $this->lastfilestamp, false);

            if ($purgecache) {
                parent::purge();
            }
            parent::set_many($filedataset);
        }
    }
}